import os, sys
import ConfigParser
from distutils.core import Command, DEBUG
from distutils.errors import DistutilsOptionError
from distutils import util, sysconfig

try:
    True
except NameError:
    # Python 2.1
    True = 1
    False = 0

CONFIG_SCHEMES = {
    'fhs_local' : {
        'pythonlibdir'  : sysconfig.get_python_lib(),
        'bindir'        : '/usr/local/bin',
        'datadir'       : '/usr/local/share/$name',
        'sysconfdir'    : '/usr/local/etc',
        'localstatedir' : '/var/local/lib/$name',
        'libdir'        : '/usr/local/lib/$name',
        'docdir'        : '/usr/local/share/doc/$fullname',
    },
    'fhs_system' : {
        'pythonlibdir'  : sysconfig.get_python_lib(),
        'bindir'        : '/usr/bin',
        'datadir'       : '/usr/share/$name',
        'sysconfdir'    : '/etc',
        'localstatedir' : '/var/lib/$name',
        'libdir'        : '/usr/lib/$name',
        'docdir'        : '/usr/share/doc/$fullname',
        },
    'posix_autoconf' : {
        'pythonlibdir'  : sysconfig.get_python_lib(),
        'bindir'        : '$exec_prefix/bin',
        'datadir'       : '$prefix/share/$name',
        'sysconfdir'    : '$prefix/etc',
        'localstatedir' : '$exec_prefix/var/$name',
        'libdir'        : '$exec_prefix/lib/$name',
        'docdir'        : '$prefix/share/doc/$fullname',
        },
    'posix_home' : {
        'pythonlibdir'  : '$prefix/lib/python',
        'bindir'        : '$prefix/bin',
        'datadir'       : '$prefix/share/$name',
        'sysconfdir'    : '$prefix/share/etc',
        'localstatedir' : '$prefix/share/var/$name',
        'libdir'        : '$prefix/lib/$name',
        'docdir'        : '$prefix/share/doc/$fullname',
        },
    'nt' : {
        'pythonlibdir'  : '$prefix/Lib/site-packages',
        'bindir'        : '$prefix/Scripts',
        'datadir'       : '$prefix/Share/$name',
        'sysconfdir'    : '$prefix/Share/Conf',
        'localstatedir' : '$prefix/Share/$name',
        'libdir'        : '$prefix/Share/$name',
        'docdir'        : '$prefix/Share/Doc/$fullname',
        },
    'other' : {
        'pythonlibdir'  : '$prefix/Lib/site-packages',
        'bindir'        : '$prefix/Scripts',
        'datadir'       : '$prefix/Share/$name',
        'sysconfdir'    : '$prefix/Share/Conf',
        'localstatedir' : '$prefix/Share/$name',
        'libdir'        : '$prefix/Share/$name',
        'docdir'        : '$prefix/Share/Doc/$fullname',
        },
    }

if sys.version < '2.2':
    CONFIG_SCHEMES['nt']['pythonlibdir'] = '$prefix'

CACHE_FILENAME = 'config.cache'

CONFIG_KEYS = ('prefix', 'exec_prefix', 'pythonlibdir', 'bindir', 'datadir',
               'sysconfdir', 'localstatedir', 'libdir', 'docdir')

class Config(Command):

    command_name = 'config'

    user_options = [
        # Select installation scheme and set base directories
        ('local', None,
         'FHS /usr/local installation scheme (default)'),
        ('system', None,
         'FHS /usr installation scheme'),
        ('prefix=', None,
         "autoconf-style installation prefix"),
        ('exec-prefix=', None,
         "autoconf-style prefix for platform-specifc files"),
        ('home=', None,
         "home directory to install under"),

        # Fine-tuning of directories
        ('pythonlibdir=', None,
         'Python 3rd-party libraries (site-packages)'),
        ('bindir=', None,
         'User executables'),
        ('datadir=', None,
         'Read-only platform-independent data'),
        ('sysconfdir=', None,
         'Read-only host-specific data'),
        ('localstatedir=', None,
         'Modifiable host-specific data'),
        ('libdir=',None,
         'Libraries for programming and packages'),
        ('docdir=', None,
         'Documentation files'),
        ]

    boolean_options = ['local', 'system']

    def initialize_options(self):
        self.cache_filename = CACHE_FILENAME
        self.cache_section = '%s-%s' % (util.get_platform(), sys.version[0:3])
        self.scheme = None
        self.config_vars = {}

        # User selectable Options
        self.local = None
        self.system = None
        self.home = None
        self.prefix = None
        self.exec_prefix = None
        self.pythonlibdir = None
        self.bindir = None
        self.datadir = None
        self.sysconfdir = None
        self.localstatedir = None
        self.libdir = None
        self.docdir = None
        return

    def finalize_options(self):
        if DEBUG: print "Config.finalize_options():"

        if os.name == 'posix':
            self.finalize_posix()
        else:
            self.finalize_other()

        # Read the configuration cache to fill in values not already set.
        parser = ConfigParser.ConfigParser()
        if os.path.exists(self.cache_filename):
            if DEBUG: print "  reading", self.cache_filename
            fp = open(self.cache_filename, 'rb')
            parser.readfp(fp)
            fp.close()

        # Use the cached configuration only if the platform specific section
        # already exists in the cache (duh!) and no options have been specified
        # by the user.
        changed = False
        if not parser.has_section(self.cache_section) or \
               self.distribution.get_option_dict(self.command_name):
            # Fill in the cache with values from the scheme and options.
            if not parser.has_section(self.cache_section):
                parser.add_section(self.cache_section)

            for key in ('prefix', 'exec_prefix'):
                value = getattr(self, key) or ''
                parser.set(self.cache_section, key, value)

            # Set the scheme directories from the options, or the default
            # value if not specified.
            scheme = CONFIG_SCHEMES[self.scheme]
            for key in scheme.keys():
                value = getattr(self, key)
                if value is None:
                    value = scheme[key]
                parser.set(self.cache_section, key, value)
            changed = True
        elif DEBUG:
            print '  using cached options'

        for name in CONFIG_KEYS:
            setattr(self, name, parser.get(self.cache_section, name))

        # Save this configuration for next run
        if DEBUG:
            print "  writing", self.cache_filename
            print '    [%s]' % self.cache_section
            for name in CONFIG_KEYS:
                print '   ', name, '=', parser.get(self.cache_section, name)

        if changed:
            fp = open(self.cache_filename, 'wb')
            parser.write(fp)
            fp.close()
        return

    def finalize_posix(self):
        # Only one of local/system/home/[prefix|exec-prefix] may be given
        # The easiest determine this is to just count the Nones
        specified_count = ((self.local is not None) +
                           (self.system is not None) +
                           (self.home is not None) +
                           ((self.prefix or self.exec_prefix) is not None))
        if specified_count > 1:
            raise DistutilsOptionError("must use only one of --local, --system,"
                                       " --home or --prefix/exec-prefix")

        if self.local or not specified_count:
            self.scheme = 'fhs_local'
        elif self.system:
            self.scheme = 'fhs_system'
        elif self.home:
            self.prefix = self.exec_prefix = os.path.expanduser(self.home)
            self.scheme = 'posix_home'
        else:
            # Either self.prefix or self.exec_prefix is specified
            if self.exec_prefix is None:
                self.exec_prefix = self.prefix
            elif self.prefix is None:
                raise DistutilsOptionError, \
                      "must not supply exec-prefix without prefix"
            self.prefix = os.path.expanduser(self.prefix)
            self.exec_prefix = os.path.expanduser(self.prefix)
            self.scheme = 'posix_autoconf'
        return

    def finalize_other(self):
        if self.local:
            self.warn("'--local' option ignored on this platform")
            self.local = None
        if self.system:
            self.warn("'--system' option ignored on this platform")
            self.system = None
        if self.exec_prefix:
            self.warn("'--exec-prefix' option ignored on this platform")
            self.exec_prefix = None
        if self.home:
            self.warn("'--home' option ignored on this platform")
            self.home = None

        if self.prefix is None:
            self.prefix = sys.prefix

        if os.name == 'nt':
            self.scheme = 'nt'
        else:
            self.scheme = 'other'
        return

    def run(self):
        pass

    def get_config_vars(self):
        config_vars = {'name' : self.distribution.get_name(),
                       'version' : self.distribution.get_version(),
                       'fullname' : self.distribution.get_fullname(),
                       'url' : self.distribution.get_url(),
                       }
        for key in CONFIG_KEYS:
            value = getattr(self, key)
            if os.name == 'posix':
                value = os.path.expanduser(value)
            value = util.convert_path(value)
            value = util.subst_vars(value, config_vars)
            config_vars[key] = value
        return config_vars
