import os
from stat import ST_MODE
from distutils.command import install_scripts

class InstallScripts(install_scripts.install_scripts):

    command_name = 'install_scripts'

    description = "install scripts (Python or otherwise)"

    user_options = [
        ('force', 'f', "force installation (overwrite existing files)"),
        ]

    boolean_options = ['force']

    def initialize_options(self):
        self.install_dir = None
        self.force = 0
        self.skip_build = None
        return

    def finalize_options (self):
        self.set_undefined_options('install',
                                   ('install_scripts', 'install_dir'),
                                   ('force', 'force'),
                                   ('skip_build', 'skip_build'),
                                   )
        return

    def run (self):
        if not self.skip_build:
            self.run_command('build_scripts')

        self.mkpath(self.install_dir)
        for file in self.get_inputs():
            self.copy_file(file, self.install_dir)
            if os.name == 'posix':
                # Set the executable bits (owner, group, and world) on
                # all the scripts we just installed.
                if self.dry_run:
                    self.announce("changing mode of %s" % file)
                else:
                    mode = ((os.stat(file)[ST_MODE]) | 0555) & 07777
                    self.announce("changing mode of %s to %o" % (file, mode))
                    os.chmod(file, mode)
        return

    def get_inputs(self):
        build_scripts = self.get_finalized_command('build_scripts')
        return build_scripts.get_outputs()
        
    def get_outputs(self):
        build_cmd = self.get_finalized_command('build_scripts')
        build_files = build_cmd.get_outputs()
        
        prefix_len = len(build_cmd.build_dir) + len(os.sep)
        outputs = []
        for file in build_files:
            outputs.append(os.path.join(self.install_dir, file[prefix_len:]))
        return outputs
        
        
